/*  smppow2.cpp  by K.Tsuru  */
//
// A sample program for Lpow(x, p) and SInteger::BitShift(p) functions.
//

const char* title =
"<<<<< A sample program for the computing of 2^p. >>>>>";

#include "sn.h"

// 2^power + plus
SLong ShowPow2(ulong power)
{
	SLong res;
	Timer timer;
	double cpu_time;
	res.FFTUsedTimes(0);	// reset
	int output=1;

	cerr << "Computing 2^" << power << " now." << endl;

	timer.Start();
	res = Lpow(2, power);	//	res = one << power; is too Ok.
	cpu_time = timer.Stop();
	cout << "CPU time " << cpu_time <<" (sec). figures = " << res.DFigures(); 
	cout << ", FFT was used " << res.FFTUsedTimes() << "times." << endl;
	if(res.DFigures()>50000L){
		cerr << "Do you want to output result ? It maybe need much time on DOS." << endl;
		cerr << "Yes(1)/No(0)=? ";
		output=GetInt();
    }
	if(output){
		cout << " 2^(" << power << ") =" << endl;
		long c = res.Puts(10);
		cout << "CPU time of 2^(" << power << ") = " << cpu_time <<"(sec). figures = ";
		cout << res.DFigures() <<" , FFT was used " <<  res.FFTUsedTimes() << " times." << endl;
		cout << "The total number of output characters including crlf, etc. = " << c << "(bytes)" << endl;
		cout << "The total number of output characters including crlf, etc. = " << res.ioCount << "(bytes)" << endl;
	    Wait();
    }
	return res;
}
// 2^p
void Pow2Test()
{
	long p;

	while(1){
		cerr << "To compute 2^p, input p (quit when p <= 0) = ? ";
		p = GetLong();
		if(p <= 0) break;
		ShowPow2(p);
	}
}

void CompareSpeedPow2_1(ulong power)
{
	double tcv1, tcv2, tsl;
	SInteger I;
	SLong s1,s2,a;
	Timer timer;

	cerr << endl << endl << "Now comparing speed of evaluating 2^" << power << " -1 by three methods." << endl;
	cout << endl << "A: Radix conversion SInteger 2^" << power <<" -1 into SLong." << endl;
	timer.Start();
	I = 1.0;	I <<= power;		I--;	//I=2^power-1
	s1 = I.ConvToDec();
	tcv1 = timer.Stop();

	cout << "B: Radix conversion SInteger 2^" << power << " into SLong and substruct one from that." << endl;
	timer.Start();
	I = 1.0;	I <<= power;	//I=2^power
	s2 = I.ConvToDec();
	s2--;
	tcv2 = timer.Stop();

	cout << "C: Using SLong function Lpow(2, " << power << ") and substruct one from that." << endl;
	timer.Start();
	a = Lpow(2, power) -1;	// same as Pow2(power)-1;
	tsl = timer.Stop();

	cout << "CPU time(sec) A:" << tcv1 << ", B:" << tcv2 << ", C:" << tsl << endl;
	if((s1==s2) && (s2==a)) cout << "All result agrees each other." << endl;
	else cerr << "An error occured" << endl;
	Wait();
}
int main()
{
	ErrorPuts(title);
	Pow2Test();
	CompareSpeedPow2_1(2000000);
	Ending();
}
